local Signal = require("lib.hump.signal")

local metatable = {
    __call = function(t, ...) 
        t.emit(...) 
    end
}

local function  defineEvent(event)
    local t = {}
    t.name = event

    t.register = function(fn)
        return Signal.register(event, fn)
    end

    t.remove = function(handle)
        Signal.remove(event, handle)
    end

    t.emit = function(...)
        Signal.emit(event, ...)
    end

    t.clear = function()
        Signal.clear(event)
    end

    
    return setmetatable(t, metatable)
end

return {
    enableInput = defineEvent("enable-input"),
    disableInput = defineEvent("disable-input"),
    transition = defineEvent("transition"),
    switchState = defineEvent("switch-state"),
    pushState = defineEvent("push-state"),
    popState = defineEvent("pop-state"),
    screenChanged = defineEvent("screen-changed"),
    scalingChanged = defineEvent("scaling-changed"),

    addEntity = defineEvent("add-entity"),
    removeEntity = defineEvent("remove-entity"),
    playMusic = defineEvent("play-music"),
    playSfx = defineEvent("play-sfx"),
    
    playerShipDestroyed = defineEvent("player-ship-destroyed"),
    addLife = defineEvent("add-life"),

    spawnExplosion = defineEvent("spawn-explosion")
}