local anim8 = require("lib.anim8")

love.graphics.setDefaultFilter("nearest", "nearest")
love.graphics.setLineStyle("rough")
fnt = love.graphics.newFont(8, "mono")
viewportY = 0

function round(x) 
    return math.floor(0.5000000001 + x)
end

function rem(x)
    return x - round(x)
end

function clamp(a, min, max)
    return math.max(min, math.min(max, a))
end

function registerTileMapCollisions(world)
    return function(tm, layer, tile)        
        local tileDef = tm:getTileDef(tile.id)
        if (tileDef and tileDef.properties.solid == true) then 
            world:add(tile, tile.x, tile.y, tile.w, tile.h)
        end
    end
end

function centreText(text, offset, top) 
    local left = (DRAW_WIDTH /2) - (fnt:getWidth(text) / 2)

    love.graphics.print(text, fnt, left + offset, top)
    
end

function updateDrawScaling()
    local scale = love.graphics.getHeight() / DRAW_HEIGHT
    scale = math.floor(scale)
    DRAW_SCALE = scale
    local width = DRAW_WIDTH * DRAW_SCALE
    local height = DRAW_HEIGHT * DRAW_SCALE

    viewportX = (love.graphics.getWidth()-width) /2
    viewportY = (love.graphics.getHeight()-height)/2

    if (viewportCanvas) then
        viewportCanvas:release()
    end
    viewportCanvas = love.graphics.newCanvas(width, height)
    Events.scalingChanged()
end

function toggleFullscreen()
    love.window.setFullscreen(not love.window.getFullscreen(), "desktop")
    Preferences.video.fullscreen = love.window.getFullscreen()
    Events.screenChanged()
end

function resizeWindow(newScale)
    local flags = {}
    flags.vsync = Preferences.video.vsync
    love.window.setMode(DRAW_WIDTH * newScale, DRAW_HEIGHT * newScale, flags) 
    Events.screenChanged()
end

function regsiterSignals()
    Events.screenChanged.register(updateDrawScaling)
end

Predicates = {
    any = function(obj)
        return true
    end,

    isPlayer = function(obj)
        return obj.isPlayer
    end,

    isPhysicsObject = function(obj)
        return obj.physics ~= nil
    end,

    belowPlayArea = function(obj)
        return obj.position.y > (PLAY_HEIGHT + obj.hitbox.h)
    end,

    abovePlayArea = function(obj)
        return obj.position.y < -obj.hitbox.h
    end
}

Damage = {
    kill = function(obj)
        return obj.health.current
    end,

    constant = function(value)
        return function(obj)
            return value
        end
    end
}

function linearMovement(nx, ny, speed)
    return function(e, dt)
        e.position.x = e.position.x + nx * speed * dt
        e.position.y = e.position.y + ny * speed * dt
    end
end


function lerp(a, b, t)
    return (a + ((b - a) * t))
end

function string.split(str, delimeter)
    local res={}
    local i=1
   
    local s,e = string.find(str,delimeter,i)
    while s ~= nil do
        table.insert(res, string.sub(str, i,s-1))
        i = s+1
        s,e = string.find(str,delimeter,i)
    end

    table.insert(res, string.sub(str,i))
    return res
end

function frames(f)
    return f * TIMESTEP
end

function turns(n)
    return math.pi*2*n
end

function newAnimation(anim, randomise, pauseAtEnd) 
    local an = anim8.newAnimation(anim.grid.g(anim.frames.x, anim.frames.y), anim.duration, pauseAtEnd and "pause" or nil)
    if randomise then
        an:gotoFrame(love.math.random(1, #an.frames))
    end
    return an
end

function normalise(x, y)
    local len = math.sqrt(x*x + y*y)
    if len > 0 then
        return x/len,y/len
    end
    return x,y
end

function drawToViewport(canvas, x,y, clear)
    if not clear then clear = true end
    love.graphics.setCanvas(viewportCanvas)
    if clear == true then
        love.graphics.clear()
    end
    love.graphics.setColor(1,1,1,1)
    love.graphics.draw(canvas,round((x or 0)*DRAW_SCALE),round((y or 0)*DRAW_SCALE),0,DRAW_SCALE,DRAW_SCALE)
    love.graphics.setCanvas()
end

function kinematicJump(jumpHeight, jumpDuration)
    local gravity = (2*jumpHeight)/(jumpDuration*jumpDuration)
    local jumpVelocity = -math.sqrt(2*gravity*jumpHeight)
    return  jumpVelocity * TIMESTEP , gravity * TIMESTEP * TIMESTEP
end

if not bit then
    bit = require("lib.bitop.funcs")
end

-- world's hackiest bmp font renderer
function bmpPrint(str, atlas, x, y)
    local offset = 0
    for i=1,#str do
        local os = 4
        local char = string.byte(str:sub(i,i))
        if char == 105 then
            os = 2
        elseif char == 32 then
            os = 3
        elseif char == 33 then 
            os = 2
        end
        if char ~= 32 then
            if char >= 65 then
                char = char - 65
            elseif char >= 48 then
                char = char-48+58
            elseif char == 33 then 
                char = 68
            end

        
            
            local cx = x + offset
            atlas:draw(char, cx, y)
            
        end
        
        offset = offset + os
    end
end