local Object = require("lib.classic")

local Input = Object:extend()

function Input:new() 
    self.defaultScope = "game"
    self.scopes = {}
end

function Input:load(inputBindings)
    local Button = require("framework.button")
    for scope, event in pairs(inputBindings) do
        local keyboard={}
        local gamepad={}
        local buttons={}
            
        for k,v in pairs(event) do
            buttons[k] = Button(k)
            for _, key in ipairs(v.keyboard) do
                keyboard[key] = buttons[k]         
            end

            for _, button in ipairs(v.gamepad) do
                gamepad[button] = buttons[k]
            end
        end
        self.scopes[scope] = {keyboard=keyboard, gamepad=gamepad, buttons=buttons}
    end
    self.defaultScope = "game"..Preferences.input.type
end

function Input:resetAll()
    for scope,_ in pairs(self.scopes) do
        self:reset(scope)
    end
end

function Input:reset(scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes[scope] then return false end
    for _,b in pairs(self.scopes[scope].buttons) do
        b:reset()
    end
end

function Input:actionStrength(event, scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes or not self.scopes[scope] then return 0 end
    local b = self.scopes[scope].buttons[event]
    if b then 
        return b:intValue()
    end
    return 0
end

function Input:isPressed(event, scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes[scope] then return false end
    local b = self.scopes[scope].buttons[event]
    if b then 
        return b.pressed
    end
    return false
end

function Input:isJustPressed(event, scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes[scope] then return false end
    local b = self.scopes[scope].buttons[event]
    if b then 
        return b:justPressed()
    end
    return false
end

function Input:isJustReleased(event, scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes[scope] then return false end
    local b = self.scopes[scope].buttons[event]
    if b then 
        return b:justReleased()
    end
    return false
end

function Input:update(scope)
    if not scope then scope = self.defaultScope end
    if not self.scopes[scope] then return false end
    for _,b in pairs(self.scopes[scope].buttons) do
        b:update()
    end
end

function Input:keypressed(k, scope)
    if not scope then scope = self.defaultScope end
    if self.scopes[scope] and self.scopes[scope].keyboard[k] then 
        self.scopes[scope].keyboard[k]:press()
        return scope.."_"..self.scopes[scope].keyboard[k].name
    end
    return k
end

function Input:keyreleased(k, scope)
    if not scope then scope = self.defaultScope end
    if self.scopes[scope] and self.scopes[scope].keyboard[k] then 
        self.scopes[scope].keyboard[k]:release()
        return scope.."_".. self.scopes[scope].keyboard[k].name
    end
    return k
end

function Input:gamepadpressed(d, k, scope)
    if not scope then scope = self.defaultScope end
    if self.scopes[scope] and self.scopes[scope].gamepad[k] then 
        self.scopes[scope].gamepad[k]:press()
        return scope.."_".. self.scopes[scope].gamepad[k].name
    end
    return k
end

function Input:gamepadreleased(d, k, scope)
    if not scope then scope = self.defaultScope end
    if self.scopes[scope] and self.scopes[scope].gamepad[k] then 
        self.scopes[scope].gamepad[k]:release()
        return scope.."_".. self.scopes[scope].gamepad[k].name
    end
    return k
end

return Input()