local Object = require("lib.classic")
local Background = require("framework.background")
local Level = Object:extend()
local Enemies = require("game.entities.enemies")

function Level:new(data)
    self.cameraPositionSpawners = {}
    self.playerPositionSpawners = {}
    self.cameraColumn = 0
    self.playerColumn = 0
    self.playerSpawn = {0, 0}
    self.overlayColor = {0,0,0,0}
    self.backgrounds = {}

    for k,v in pairs(data) do 
        self[k] = v
    end
end

function Level:start()

end

function Level:load()
    for _, obj in ipairs(self.tilemap.objects) do
        if obj.type == "player_spawn" then
            self.playerSpawn[1] = obj.x
            self.playerSpawn[2] = obj.y
        elseif obj.type ~= "position_trigger" and not obj.properties.player_pos then
            local col = math.floor(obj.x / 4)

            if not self.cameraPositionSpawners[col] then
                self.cameraPositionSpawners[col] = {}
            end
            local spawnersForCol = self.cameraPositionSpawners[col]
            table.insert(spawnersForCol, obj)
        end
    end

    for _, obj in ipairs(self.tilemap.objects) do
        if obj.properties.player_pos and obj.properties.player_pos.id ~= 0 then
            local pos = self.tilemap.objectsById[obj.properties.player_pos.id]
            if pos then
                local col = math.floor(pos.x / 4)

                if not self.playerPositionSpawners[col] then
                    self.playerPositionSpawners[col] = {}
                end
                local spawnersForCol = self.playerPositionSpawners[col]
                table.insert(spawnersForCol, obj)
            end
         end
     end

end

function Level:update(gameLogic, dt)
    local cam = gameLogic.camera
    local currentCameraColumn = math.floor((cam.pos.x + cam.half_width)/4) 

    if currentCameraColumn ~= self.cameraColumn then
        if currentCameraColumn > self.cameraColumn and self.cameraPositionSpawners[currentCameraColumn] then
            for _, spawner in ipairs(self.cameraPositionSpawners[currentCameraColumn]) do
                if Enemies[spawner.type] then
                    local enemy = Enemies[spawner.type](spawner.x, spawner.y-4.05, spawner.properties)
                    enemy.sid = spawner.id
                    gameLogic:addEnemy(enemy)
                end
            end
        end

        self.cameraColumn = currentCameraColumn
    end

    local currentPlayerColumn = math.floor(gameLogic.player.position.x / 4)

    if currentPlayerColumn ~= self.playerColumn then
        if  self.playerPositionSpawners[currentPlayerColumn] then
            for _, spawner in ipairs(self.playerPositionSpawners[currentPlayerColumn]) do
                if Enemies[spawner.type] then
                    local enemy = Enemies[spawner.type](spawner.x, spawner.y-4.05, spawner.properties)
                    enemy.sid = spawner.id
                    gameLogic:addEnemy(enemy)
                end
            end
        end

        self.playerColumn = currentPlayerColumn
    end
end

function Level:draw(gameLogic)
    local x = gameLogic.camera:getRect()
    for i, bg in ipairs(self.backgrounds) do
        bg:draw(x)
    end
    love.graphics.setColor(self.overlayColor)
    love.graphics.rectangle("fill", 0,0,DRAW_WIDTH*DRAW_SCALE,DRAW_HEIGHT*DRAW_SCALE)
    love.graphics.setColor(1,1,1,1)

    local cam = gameLogic.camera
    for _, layer in ipairs(self.tilemap.layerNames) do
        local  l = self.tilemap.layersByName[layer]
        if layer ~= "collision" and l.parallaxx <= 1 then
            cam:attach(l.parallaxx, 1)
            local x,y,w,h = gameLogic.camera:getRect()
            self.tilemap:drawLayer(layer, x*l.parallaxx,y,w,h)
            cam:detach()
        end
    end
end

function Level:checkEndCondition(gameLogic)
    local pos = math.ceil(gameLogic.player.position.x / self.tilemap.tilewidth)
    return pos == self.tilemap.width
end

function Level:drawDebug(gameLogic)
    if debug == true then
        self.tilemap:drawLayer("collision", gameLogic.camera:getRect())
    end
end

function Level:drawFront(gameLogic)
    
    for _, layer in ipairs(self.tilemap.layerNames) do
        if layer ~= "collision" and self.tilemap.layersByName[layer].parallaxx > 1 then
            cam:attach(l.parallaxx, 1)
            local x,y,w,h = gameLogic.camera:getRect()
            self.tilemap:drawLayer(layer, x*l.parallaxx,y,w,h)
            cam:detach()
        end
    end
end

return Level

 