local Object = require("lib.classic")


local Sensors = Object:extend()
function Sensors:new(data) 
    self.all = {}
    self.left= {}
    self.right = {}
    self.up = {}
    self.down = {}

    local dirs = {
        left ={x=-1,y=0},
        right ={x=1,y=0},
        up ={x=0,y=-1},
        down ={x=0,y=1},
    }

    for k,v in pairs(data) do
        if dirs[k] then
            local d = dirs[k]
            local col = self[k]

            for i,s in ipairs(v) do
                s.dx = d.x
                s.dy = d.y
                table.insert(self.all, s)
                table.insert(col, s)
            end

        end
    end
end

function Sensors:update(x, y, tilemap, sensors)
    local colliding = false
    for _, ps in ipairs(sensors) do
        local cx = x + ps.x
        local cy = y + ps.y

        ps.colliding = false
        ps.depth = 0
        ps.tileId = -1
        ps.tileHeightAtPosition = 0
        ps.sx = ps.x
        ps.sy = ps.y

        local tile = tilemap:getTileFromWorld("collision", cx, cy)
        if (tile ~= nil and (tile.id > 0)) then
            ps.colliding = true
            ps.tileId = tile.id
            ps.tx = tile.x
            ps.ty = tile.y
            ps.tileHeightAtPosition = cy

            if ps.dx ~= 0 then
                if ps.dx == -1 then
                    ps.depth = math.abs((tile.w+tile.x) - cx) 
                else
                    ps.depth = math.abs(cx  - tile.x) + 0.001
                end
            elseif ps.dy ~= 0 then
                if ps.dy == -1 then
                    ps.depth = math.abs((tile.h+tile.y) - cy) 
                else
                    ps.depth = math.abs(cy  - tile.y) + 0.001
                end     
            end
        end
        colliding = colliding or ps.colliding
    end
    
    return colliding
end

function Sensors:reset(sensors)
    for _, ps in ipairs(sensors) do
        ps.colliding = false
        ps.depth = 0
        ps.tileId = -1
        ps.tileHeightAtPosition = 0
        ps.sx = ps.x
        ps.sy = ps.y
    end
end

function Sensors:getCollisionInfo(sensors)
    local ej = 0
    local tileId = -1
    local tx = 0
    local ty = 0
    local sx = 0
    local sy = 0
    for _,sensor in ipairs(sensors) do
        if (sensor.depth ~= nil and sensor.depth > ej) then
            ej = sensor.depth
            tileId = sensor.tileId
            tx = sensor.tx
            ty = sensor.ty
            sx = sensor.sx
            sy = sensor.sy
        end
    end

    return ej, tileId, tx, ty, sx, sy
end

return Sensors