local Object = require("lib.classic")

local SoundManager = Object:extend()

function SoundManager:new(preferences)
    self.timer = Timer.new()
    self.masterVolume = (preferences.masterVolume/10) or 1
    self.musicVolume = (preferences.musicVolume/10) or 1
    self.sfxVolume = (preferences.sfxVolume/10) or 1
    self.music = nil

    Events.playMusic.register(function(music) self:playMusic(music) end)
    Events.playSfx.register(function(sfx) self:playSfx(sfx) end)
end

function SoundManager:update(dt)
    self.timer:update(dt)
end

function SoundManager:playMusic(def, vol)
    if self.music and self.music:isPlaying() then
        self.music:stop()
    end

    self.music = def.source
    self.music:setLooping(def.looping or true)
    self.music:setVolume(self.masterVolume * self.musicVolume * (def.volume or 1) )
    self.music:play()
end

function SoundManager:stopMusic()
    if self.music then
        self.music:stop()
    end
end

function SoundManager:playSfx(source, volume)
    -- local source = def.source:clone()
    source:setVolume(self.masterVolume * self.sfxVolume * (volume or 1))
    love.audio.play(source)
end

function SoundManager:fadeOut(len)
    local o = {t = 0, volume = self.music:getVolume()}
    self.timer:clear()
    self.timer:during(len, function(dt)
        o.t = o.t + dt
        local t = 1 - (o.t / len)
        self.music:setVolume(t * o.volume)
        if self.music:getVolume() == 0 then
            self.music:stop()
        end
    end)
end

function SoundManager:fadeIn(def, len)
    self:playMusic(def)
    self.music:setVolume(0)
    local o = {t = 0, volume = self.masterVolume * self.sfxVolume * (def.volume or 1)}
    self.timer:during(len, function(dt)
        o.t = o.t + dt
        local t = (o.t / len)
        self.music:setVolume(t * o.volume)
        if self.music:getVolume() == 0 then
            self.music:stop()
        end
    end)
end
return SoundManager