local Object = require("lib.classic")

local StateMachine = Object:extend()

function StateMachine:new(states)
    self.states = states
    self.current = nil
end

function StateMachine:exitCurrent(...)
    local current = self.current
    local state = self.states[current]
    if state and state.exit then
        state.exit(self,...)
    end
end

function StateMachine:enterCurrent(...)
    local current = self.current
    local state = self.states[current]
    if state and state.enter then
        state.enter(self, ...)
    end
end

function StateMachine:update(...)
    local current = self.current
    local state = self.states[current]
    if state and state.update then
        state.update(self,...)
    end
end


function StateMachine:switch(newState, ...)
    self:exitCurrent(...)
    self.current = newState
    self:enterCurrent(...)
end



return StateMachine