local Object = require("lib.classic")

local TextureAtlas = Object:extend()

function TextureAtlas:new(texture, numCols, numRows, regionX, regionY, regionW, regionH)
    self.texture = texture
    self.numCols = numCols
    self.numRows = numRows
    self.quads = {}

    regionX = regionX or 0
    regionY = regionY or 0
    regionW = regionW or texture:getWidth()
    regionH = regionH or texture:getHeight()

    local w,h = texture:getWidth(), texture:getHeight()
    local cw,ch = regionW/numCols, regionH/numRows

    for y=0,numRows-1 do
        for x=0,numCols-1 do
            local q = love.graphics.newQuad(regionX + (x*cw), regionY + (y*ch), cw, ch, w, h)
            table.insert(self.quads, q)
        end
    end
end

function TextureAtlas:draw(id, x, y, ox, oy, r, sx, sy)
    r = r or 0
    sx = sx or 1
    sy = sy or 1
    ox = ox or 0
    oy = oy or 0
    local q = self.quads[id+1]
    if q then
        love.graphics.draw(self.texture, q, round(x * DRAW_SCALE), round(y * DRAW_SCALE), r,  DRAW_SCALE * sx,  DRAW_SCALE * sy, ox, oy)
    end
end

function TextureAtlas:drawByCoords(tx, ty,  x, y, r, sx, sy, ox,oy)
    local id = ((ty-1) * self.numCols) + tx-1
    self:draw(id, x, y, r, sx, sy, ox, oy)
end

function TextureAtlas:get(id)
    return self.quads[id]
end

return TextureAtlas