local Entity = require("framework.entity")
local StateMachine = require("framework.state_machine")
local Flame = function(x, y, props)
    return Entity(x, y, {
        invincibility = 99999,
        stats = {hp = 1000},
        static = true,
        t = 0,
        extents = {w=2, h=3},
        atlas = Assets.atlases.flame,

        
        update = function(self)
           self.position.y = self.position.y + 1 
        end,
    
        init = function(rn, gameLogic)
            rn:playAnimation({
                {id=0, ox=4, oy=4, t=3},
                {id=1, ox=4, oy=4, t=3}
            })
        end,
    })
end

local Animations = {
    idle = {
        {id=0, ox=16, oy=16, t=1000}
    },
    telegraph = {
        {id=1, ox=16, oy=16, t=1000}
    },
    teleport = {
        {id=3, ox=16, oy=16, t=2},
        {id=4, ox=16, oy=16, t=2},
        {id=5, ox=16, oy=16, t=2},
        {id=6, ox=16, oy=16, t=2},
        {id=7, ox=16, oy=16, t=2},
    },
    reappear = {
        {id=7, ox=16, oy=16, t=2},
        {id=6, ox=16, oy=16, t=2},
        {id=5, ox=16, oy=16, t=2},
        {id=4, ox=16, oy=16, t=2},
        {id=3, ox=16, oy=16, t=2},
    },
    damaged = {
        {id=2, ox=16, oy=16, t=90},
    }
}

local points = {
    {x=56, y=48},
    {x=6, y=48},
    {x=6, y=24},
    {x=56, y=24}
}

local BossStates = {
    idle = {
        enter = function(sm, boss)
            boss:playAnimation(Animations.idle)
            boss.t = 0
            boss.invincibility = 0
        end,

        update = function(sm, boss, dt)
            if boss.t > 120 then
                sm:switch("teleport", boss)
            end

        end
    },

    teleport = {
        enter = function(sm, boss)
            boss:playAnimation(Animations.teleport)
            boss.t = 0
            boss.invincibility = 10000
        end,
        update = function(sm, boss, dt)
            if boss.t >= 10 then
                sm:switch("attack", boss)
            end
        end

    },

    reappear = {
        enter = function(sm, boss, position)
            if position == nil then
                position = points[math.random(#points)]
            end
            boss.visible = true
            boss.position.x = position.x
            boss.position.y = position.y
            boss:playAnimation(Animations.reappear)
            boss.t = 0
            boss.invincibility = 10000
        end,

        update = function(sm, boss, dt)
            if boss.t >= 10 then
                sm:switch("idle", boss)
            end
        end
    },

    attack = {
        enter = function(sm, boss)
            boss.visible = false
            boss.position.x = 32
            boss.position.y = -32
           
            boss.t = 0
        end,

        update = function(sm, boss, gameLogic)
            local firePeriod = 30
            if boss.stats.hp <= 5 then
                firePeriod = 30
            elseif boss.stats.hp <= 3 then
                firePeriod = 10
            end

            if boss.t % firePeriod == 0 then
                local player = gameLogic.player
                gameLogic:addEnemy(Flame(player.position.x, -8))
            end

            if boss.t == 4*60 then
                sm:switch("reappear", boss)
            end
        end
    },

    damaged = {
        enter = function(sm, boss)
            boss:playAnimation(Animations.damaged)
            boss.t = 0
            boss.invincibility = 10000
        end,

        exit = function(sm, boss)
            boss.visible = true
        end,

        update = function(sm, boss, dt)
            boss.visible = not boss.visible
            if boss.t == 90 then
                sm:switch("teleport", boss)
            end
        end
    },

    dying = {
        enter = function(sm, boss)
            boss.stats.hp = 1
            boss.invincibility = 10000
            boss:playAnimation(Animations.damaged)
            boss.t = 0
        end,

        update = function(sm, boss, gameLogic)
            boss.visible = not boss.visible

            if boss.t % 10 == 0 then
                table.insert(gameLogic.effects, effect(boss.position.x+math.random(-boss.extents.w,boss.extents.w), boss.position.y+math.random(-boss.extents.h, boss.extents.h), flashEffect))
                Assets.sfx.enemyDie:play()
            end

            if boss.t == 180 then
                gameLogic:removeEnemy(boss)
            end
        end
    }
}

local function updateBoss(boss, gameLogic, dt)
    boss.t = (boss.t+1)%256
    boss.stateMachine:update(boss, gameLogic, dt)

end

return function(x, y, props)
    
    local stats = {}
    stats.jumpVelocity, stats.gravity = kinematicJump(3.4*CELL_SIZE, 0.25)
    stats.groundSpeed = 0.65
    stats.maxFallSpeed = 2.5
    stats.hp = 6
    
    local startPoint = points[1]

    return Entity(startPoint.x, startPoint.y, {
        isBoss = true,
        invincibility = 0,
        t = 0,
        stats = stats,
        stateMachine = StateMachine(BossStates, "idle"),
        extents = {w=6, h=8},
        atlas = Assets.atlases.boss,

        
        update = updateBoss,
    
        init = function(boss, gameLogic)
            boss.stateMachine:switch("reappear", boss, points[1])
        end,

        onTakeDamage = function(boss, gameLogic)
            if boss.stateMachine.current == "idle" then
                boss.stateMachine:switch("damaged", boss)
            end
        end,
    
        onDeath = function(boss, gameLogic)
            boss.stateMachine:switch("dying", boss)
            return true
            -- for _, e in ipairs(gameLogic.enemies) do
                
            --     gameLogic:kill(e)
            -- end
        end

    })
end