local Sensors = require("framework.sensors")
local Entity = require("framework.entity")
local Physics = require("game.physics")

local Animations = {
    running = {
        {id=0, ox=8, oy=9, t=4},
        {id=1, ox=8, oy=9, t=4},
        {id=2, ox=8, oy=9, t=4},
        {id=3, ox=8, oy=9, t=4},
        {id=2, ox=8, oy=9, t=4},
        {id=1, ox=8, oy=9, t=4},
    }
}

local function updateRedNinja(rn, gameLogic, dt)
    rn.velocity.x = rn.dir * rn.stats.groundSpeed
    Physics.applyGravity(rn)
    local result = Physics.applyMovement(rn, gameLogic, dt)
    


    if bit.band(result, COLLIDE_DOWN) ~= COLLIDE_DOWN then
        if rn.grounded == true then
            rn.velocity.y = rn.stats.jumpVelocity
            rn.grounded = false
        end
    else
        if rn.reverseOnWallHit == true and bit.band(result, COLLIDE_LEFT+COLLIDE_RIGHT) ~= 0 then
            rn.dir = rn.dir * -1
        end
        rn.grounded = true
    end

    if rn.grounded == true then
        local toCheck = nil
        if rn.dir == 1 then
            toCheck = rn.whiskers.right
        elseif rn.dir == -1 then
            toCheck = rn.whiskers.left
        end
        rn.whiskers:update(rn.position.x, rn.position.y, gameLogic.tilemap, toCheck)
        for i,w in ipairs(toCheck) do
            if w.colliding == true then
                rn.velocity.y = rn.stats.jumpVelocity
                rn.grounded = false
            end
        end

    end

    rn.flipH = rn.dir == -1
end

return function(x, y, props)
    local stats = {}
    stats.jumpVelocity, stats.gravity = kinematicJump(3.4*CELL_SIZE, 0.25)
    stats.groundSpeed = 0.65
    stats.maxFallSpeed = 2.5
    stats.hp = 1
    
    return Entity(x, y, {
        dir = props.dir or -1, 
        reverseOnWallHit = props.reverseOnWallHit,
        t = 0,
        stats = stats,
        -- stateMachine = StateMachine(PlayerStates, "idle"),
        extents = {w=3, h=4},
        atlas = Assets.atlases.redNinja,

        whiskers = Sensors({
            left = {
                {x = -7, y= 0}
            },
            right = {
                {x=7, y=0}
            }
        }),
        sensors = Sensors({
            left = {
              {x=-2, y=4},   
            },
            right = {
              {x=-0, y=4},   
            },
            down = {
                {x=-2, y=4},   
                {x=-0, y=4},   
            },
        }),
    
        init = function(rn, gameLogic)
            rn:playAnimation(Animations.running)
        end,
    
        update = updateRedNinja,

    })
end