local Sensors = require("framework.sensors")
local Entity = require("framework.entity")
local Physics = require("game.physics")

local Animations = {
    running = {
        {id=0, ox=8, oy=9, t=4},
        {id=1, ox=8, oy=9, t=4},
        {id=2, ox=8, oy=9, t=4},
        {id=3, ox=8, oy=9, t=4},
        {id=2, ox=8, oy=9, t=4},
        {id=1, ox=8, oy=9, t=4},
    }
}

local function updateWalkingBaddie(rn, gameLogic, dt)
    rn.velocity.x = rn.dir * rn.stats.groundSpeed
    Physics.applyGravity(rn)
    local result = Physics.applyMovement(rn, gameLogic, dt)
    
    if bit.band(result, COLLIDE_DOWN) ~= COLLIDE_DOWN then
        rn.grounded = false
    else
        rn.grounded = true
    end

    if bit.band(result, COLLIDE_LEFT+COLLIDE_RIGHT) ~= 0 then
        rn.dir = rn.dir *- 1
    end

    if rn.stayOnPlatform == true then
        local allColliding = true
        if rn.grounded == true then
        rn.sensors:update(rn.position.x, rn.position.y+2, gameLogic.tilemap, rn.sensors.down)
            for i,w in ipairs(rn.sensors.down) do
                allColliding = allColliding and w.colliding
                
            end

        end

        if allColliding == false then
            rn.dir = rn.dir * -1
        end
    end
    rn.flipH = rn.dir == -1
end

return function(x, y, props)
    local stats = {}
    _, stats.gravity = kinematicJump(3.4*CELL_SIZE, 0.25)
    stats.groundSpeed = 0.2
    stats.maxFallSpeed = 2.5
    stats.hp = 1
    
    return Entity(x, y, {
        dir = props.dir or -1, 
        stayOnPlatform = props.stay_on_platform,-- ~= nil and props.stay_on_platform or true,
        t = 0,
        stats = stats,
        extents = {w=3, h=4},
        atlas = Assets.atlases.enemy1,

        sensors = Sensors({
            left = {
              {x=-2, y=4},   
            },
            right = {
              {x=-0, y=4},   
            },
            down = {
                {x=-2, y=4},   
                {x=-0, y=4},   
            },
        }),
    
        init = function(rn, gameLogic)
            rn:playAnimation(Animations.running)
        end,
    
        update = updateWalkingBaddie,

    })
end