local tileCollisionInfo = {
    [5] = {
        ground = COLLIDE_LEFT+COLLIDE_RIGHT+COLLIDE_UP+COLLIDE_DOWN,
        air = COLLIDE_LEFT+COLLIDE_RIGHT+COLLIDE_UP+COLLIDE_DOWN,
    },
    [6] = {
        ground = COLLIDE_DOWN,
        air = COLLIDE_DOWN,
    },
    [7] = {
        ground = 0,
        air = COLLIDE_RIGHT,
    },
    [8] = {
        ground = 0,
        air = COLLIDE_LEFT,
    },
    [9] = {
        ground = COLLIDE_DOWN,
        air = COLLIDE_RIGHT+COLLIDE_DOWN,
    },
    [10] = {
        ground = COLLIDE_DOWN,
        air = COLLIDE_LEFT+COLLIDE_DOWN,
    },
    [11] = {
        ground = 0,
        air = COLLIDE_LEFT+COLLIDE_RIGHT,
    },
    [12] = {
        ground = COLLIDE_DOWN,
        air = COLLIDE_LEFT+COLLIDE_RIGHT+COLLIDE_DOWN,
    },
}

local function applyGravity(player)
    player.velocity.y = math.min(player.stats.maxFallSpeed, player.velocity.y + player.stats.gravity)
end

local function applyMovement(player, gameLogic)
    local vel = player.velocity
    local pos = player.position
    local sensors= player.sensors

    local velx = vel.x 
    local vely =  vel.y 

    local nx = pos.x + velx
    
    sensors:reset(sensors.all)

    local result = 0

    if vel.x < 0 then
        if sensors:update(nx, pos.y, gameLogic.tilemap, sensors.left) then
            local ej,tid,tx,ty,sx,sy = sensors:getCollisionInfo(sensors.left)
            local col = player.inAir and tileCollisionInfo[tid].air or tileCollisionInfo[tid].ground
            if col and bit.band(col, COLLIDE_LEFT) ~= 0 and tx+3 < pos.x+sx then
                nx = nx + ej 
                result = result + COLLIDE_LEFT
            end
        end
    elseif vel.x > 0 then
        if sensors:update(nx, pos.y, gameLogic.tilemap, sensors.right) then
            local ej,tid,tx,ty,sx,sy = sensors:getCollisionInfo(sensors.right)
            local col = player.inAir and tileCollisionInfo[tid].air or tileCollisionInfo[tid].ground
            if col and  bit.band(col, COLLIDE_RIGHT) ~= 0 and tx > pos.x+sx then
                nx = nx - ej 
                result = result + COLLIDE_RIGHT
            end
        end
    end

    
    local ny = pos.y + vely

    if vel.y < 0 then
        if sensors:update(nx, ny, gameLogic.tilemap, sensors.up) then
            local ej,tid = sensors:getCollisionInfo(sensors.up)
            local col = tileCollisionInfo[tid].ground
            if col and bit.band(col, COLLIDE_UP) ~= 0 then
                ny = ny + ej 
                vel.y =0
                result = result + COLLIDE_UP
            end
        end
    elseif vel.y > 0 then
        if sensors:update(nx, ny, gameLogic.tilemap, sensors.down) then
            local ej,tid,tx,ty,sx,sy = sensors:getCollisionInfo(sensors.down)
            local col = tileCollisionInfo[tid].ground
            
            if col and bit.band(col, COLLIDE_DOWN) ~= 0 and ty > pos.y+sy then
                ny = ny - ej
                vel.y = 0
                
                result = result+COLLIDE_DOWN
            end
        end
    end

    pos.x = nx
    pos.y = ny

    return result
end

return {
    applyMovement = applyMovement,
    applyGravity = applyGravity
}