require("consts")
require("framework.helpers")

debug = false
Events = require("events")
Gamestate = require("lib.hump.gamestate")
UI = require("ui.user_interface")

Events.switchState.register(function(state,...)
    local params = {...}
    Gamestate.switch(state, ...) 
end)
Events.pushState.register(Gamestate.push)
Events.popState.register(Gamestate.pop)

anim8 = require("lib.anim8")
Timer = require("lib.hump.timer")
inputEnabled = true

local overlay = {
    alpha = 0,
    running = false
}

Events.transition.register(function(t,...)
    params = {...}
      
    Timer.script(function(wait)
        Events.disableInput()
        overlay.alpha = 0
        local d = t /2
        Timer.tween(d, overlay, {alpha=1}, "linear")
        wait(d)
        Events.switchState(unpack(params))
        Events.enableInput()
        Timer.tween(d, overlay, {alpha=0}, "linear")
    end)
end)


Events.enableInput.register(function()
    inputEnabled = true
    Input:resetAll()
end)


Events.disableInput.register(function()
    inputEnabled = false
    Input:resetAll()
end)


function love.load()
    regsiterSignals()

    Assets = require("game.assets")
    States = {
        LevelIntro = require("states.level_intro_state"),
        Game = require("states.game_state"),
        Title = require("states.title_state"),
        GameOver = require("states.game_over_state"),
        GameBeat = require("states.game_beat_state")
    }
    Levels = require("game.levels")
    
    Gamestate.registerEvents()

    Preferences = require("preferences")
    Preferences:load()
    resizeWindow(Preferences.video.scale)
    if (Preferences.video.fullscreen) then
        toggleFullscreen()
    end

    drawCanvas = love.graphics.newCanvas(DRAW_WIDTH+CELL_SIZE, DRAW_HEIGHT+CELL_SIZE)
    
    Input = require("framework.input")
    Input:load(Preferences.inputBindings)

    SoundManager = require("framework.sound_manager")(Preferences.audio)
    Events.switchState(States.Title)
 
end


function love.update(dt)
    Timer.update(dt)
    SoundManager:update(dt)
    Gamestate.update(dt)
end

function love.draw()
    -- draw game/menus
    Gamestate.draw()

    -- draw to window
    love.graphics.setCanvas()
    love.graphics.clear()
    love.graphics.draw(viewportCanvas, viewportX, viewportY)

    love.graphics.setColor(0,0,0, overlay.alpha)
    love.graphics.rectangle("fill", 0, 0, love.graphics.getWidth(), love.graphics.getHeight())
    love.graphics.setColor(1,1,1,1)
end

