local preferences = {}

preferences.video = {}
preferences.video.vsync = 1
preferences.video.scale = 8
preferences.video.fullscreen = false

preferences.audio = {}
preferences.audio.masterVolume = 10
preferences.audio.musicVolume = 7
preferences.audio.sfxVolume = 10

preferences.input = {}
preferences.input.type="A"

preferences.inputBindings = {}
preferences.inputBindings.gameA={}
preferences.inputBindings.gameA.up = {keyboard={"w", "up"}, gamepad={"dpup"}}
preferences.inputBindings.gameA.down = {keyboard={"s", "down"}, gamepad={"dpdown"}}
preferences.inputBindings.gameA.left = {keyboard={"a", "left"}, gamepad={"dpleft"}}
preferences.inputBindings.gameA.right = {keyboard={"d", "right"}, gamepad={"dpright"}}
preferences.inputBindings.gameA.jump = {keyboard={"j", "x"}, gamepad={"a"}}
preferences.inputBindings.gameA.attack = {keyboard={"k", "c"}, gamepad={"x"}}
preferences.inputBindings.gameA.pause = {keyboard={"escape"}, gamepad={"start"}}

preferences.inputBindings.gameB={}
preferences.inputBindings.gameB.up = {keyboard={"up"}, gamepad={"dpup"}}
preferences.inputBindings.gameB.down = {keyboard={"down"}, gamepad={"dpdown"}}
preferences.inputBindings.gameB.left = {keyboard={"left"}, gamepad={"dpleft"}}
preferences.inputBindings.gameB.right = {keyboard={"right"}, gamepad={"dpright"}}
preferences.inputBindings.gameB.jump = {keyboard={"x"}, gamepad={"a"}}
preferences.inputBindings.gameB.attack = {keyboard={"c"}, gamepad={"x"}}
preferences.inputBindings.gameB.pause = {keyboard={"escape"}, gamepad={"start"}}

preferences.inputBindings.menu={}
preferences.inputBindings.menu.up = {keyboard={"up", "w"}, gamepad={"dpup"}}
preferences.inputBindings.menu.down = {keyboard={"down", "s"}, gamepad={"dpdown"}}
preferences.inputBindings.menu.left = {keyboard={"left", "a"}, gamepad={"dpleft"}}
preferences.inputBindings.menu.right = {keyboard={"right", "d"}, gamepad={"dpright"}}
preferences.inputBindings.menu.accept = {keyboard={"j", "x", "space", "return"}, gamepad={"a"}}
preferences.inputBindings.menu.cancel = {keyboard={"k", "c", "escape"}, gamepad={"x"}}

function preferences:getPreference(key)
    local els=string.split(key, "%.")
    local t = self
    for i=1,#els do
        local tt = t[els[i]]
        if tt ~= nil then
            t = tt
        else 
            return nil
        end
    end

    return t
end

function preferences:setPreference(key, value)
    local els=string.split(key, "%.")
    local t = self
    for i=1,#els-1 do
        local tt = t[els[i]]
        if tt ~= nil then
            t = tt
        else 
            t = {}
            t[els[i]] = t

        end
    end
    
    t[els[#els]] = value
end

function preferences:persist()
    local f = love.filesystem.newFile("conf.ini")
    f:open("w")

    local keysToPersist = {"video", "audio", "input"}

    for _,ktp in ipairs(keysToPersist) do
        local key = ktp
        local t = self
        for k,v in pairs(t[key]) do
            f:write(key .. "." .. k .. "=" .. tostring(v))
            f:write("\n")
        end
    end
end

function preferences:load()
    if not love.filesystem.getInfo then return end
    local info = love.filesystem.getInfo("conf.ini") 
    
    if not info then
        self:persist()
        self.firstGame = true
    end
    
    local f = love.filesystem.newFile("conf.ini")
    f:open("r")

    for line in f:lines() do
        local d = string.split(line, "=")

        local value = d[2]
        if tonumber(value) ~= nil then
            value = tonumber(value)
        elseif value == "true" then 
            value = true
        elseif value == "false" then 
            value = false
        end
        
        self:setPreference(d[1], value)
    end
end

return preferences