local Object = require("lib.classic")

local BaseState = Object:extend()

function BaseState:new()
end

function BaseState:enter()
    Input:reset(self.inputState, self.inputState or Input.defaultScope)
end

function BaseState:update(dt)
    Input:update(self.inputState, self.inputState or  Input.defaultScope)
end

function BaseState:keyreleased(key)
    if inputEnabled then
        local event = Input:keyreleased(key, self.inputState or Input.defaultScope)
        self:callback("onReleased_" .. event)
    end

    if key == "rshift" then
        if not love.filesystem.getInfo("screenshots") then
            love.filesystem.createDirectory("screenshots")
        end
        love.graphics.captureScreenshot("screenshots/screenshot_" .. os.time() .. ".png")
    end
end

function BaseState:keypressed(key)
    if inputEnabled then
        local event = Input:keypressed(key, self.inputState or  Input.defaultScope)
        self:callback("onPressed_" .. event)
    end

    if key == "space" then 
        love.graphics.captureScreenshot(os.time() .. ".png")
    end
end


function BaseState:gamepadpressed(j, button)
    if inputEnabled then
        local event = Input:gamepadpressed(j, button, self.inputState or  Input.defaultScope)
        self:callback("onPressed_" .. event)
    end
end


function BaseState:gamepadreleased(j, button)
    if inputEnabled then
        local event = Input:gamepadreleased(j, button, self.inputState or  Input.defaultScope)
        self:callback("onReleased_" .. event)
    end
end

function BaseState:callback(name)
    if self[name] then
        self[name](self)
    end
end

return BaseState