local Object = require("lib.classic")

local ControlPreferenceMenu = Object:extend()

local options = {
    {value="gameA", label="Type A"}, 
    {value="gameB", label="Type B"}
}

function ControlPreferenceMenu:new(onSubmit)
    self.onSubmit = onSubmit
    local currentScope = Input.defaultScope
    self.selected = currentScope == "gameA" and 1 or (currentScope == "gameB" and 2)
end

function ControlPreferenceMenu:onPressed_menu_left()
    self.selected = self.selected - 1
    if self.selected < 1 then 
        self.selected = 2
    end
end

function ControlPreferenceMenu:onPressed_menu_right()
    self.selected = self.selected + 1
    if self.selected > 2 then 
        self.selected = 1
    end
end

function ControlPreferenceMenu:onPressed_menu_accept()
    Input.defaultScope = options[self.selected].value
    Preferences.input.type = string.sub(Input.defaultScope, -1)
    Preferences:persist()
    self.onSubmit()
end

function ControlPreferenceMenu:draw()
    love.graphics.setCanvas(States.Game.hudCanvas)
    love.graphics.clear()
    centreText("Choose control scheme", 0, 4)

    centreText("< Keyboard " .. options[self.selected].label .. " >", 0, 16+8)
    love.graphics.print("Movement: ",fnt,85 , 32+8)
    love.graphics.print("Fire Primary: ",fnt,85 , 32+16+8)
    love.graphics.print("Fire Secondary: ",fnt,85 , 32+16+16+8)

    centreText("XBox Gamepad ", 0, 32+16+16+24+8)
    love.graphics.print("Movement: ",fnt,85 , 32+16+16+24+16+8)
    love.graphics.print("Fire Primary: ",fnt,85 , 32+16+16+24+16+16+8)
    love.graphics.print("Fire Secondary: ",fnt,85 , 32+16+16+24+16+16+16+8)

    local scope = options[self.selected].value
    local dirs = {
        {Preferences.inputBindings[scope].up.keyboard[1],Preferences.inputBindings[scope].up.gamepad[1]},
        {Preferences.inputBindings[scope].left.keyboard[1],Preferences.inputBindings[scope].left.gamepad[1]},
        {Preferences.inputBindings[scope].down.keyboard[1],Preferences.inputBindings[scope].down.gamepad[1]},
        {Preferences.inputBindings[scope].right.keyboard[1],Preferences.inputBindings[scope].right.gamepad[1]},
    }

    local actions = {
        {Preferences.inputBindings[scope].shoot_primary.keyboard[1],Preferences.inputBindings[scope].shoot_primary.gamepad[1]},
        {Preferences.inputBindings[scope].shoot_secondary.keyboard[1],Preferences.inputBindings[scope].shoot_secondary.gamepad[1]},
    }
    
    for i,v in ipairs(dirs) do
        love.graphics.draw(Assets.images.icons.keyboard, Assets.icons.keyboard[v[1]], 162+16*i, 32+8)
        love.graphics.draw(Assets.images.icons.gamepad, Assets.icons.gamepad[v[2]], 162+16*i, 32+16+16+24+16+8)
        i=i+1
    end
    love.graphics.draw(Assets.images.icons.keyboard, Assets.icons.keyboard[actions[1][1]], 162+16, 32+16+8)
    love.graphics.draw(Assets.images.icons.keyboard, Assets.icons.keyboard[actions[2][1]], 162+16, 32+16+16+8)
    
    love.graphics.draw(Assets.images.icons.gamepad, Assets.icons.gamepad[actions[1][2]], 162+16, 32+16+16+24+16+16+8)
    love.graphics.draw(Assets.images.icons.gamepad, Assets.icons.gamepad[actions[2][2]], 162+16, 32+16+16+24+16+16+16+8)
    
end 

return ControlPreferenceMenu